          module diagcomp

!	***********************************************************************
!		This is a reduced version of the module dblcomp_mod, containing	
!		a linear statistical comparison against a uniform random number
! 		distribution.		
!		The 1-dimensional, 1-component array disch() keeps track of		
!		whether a discharge breakdown has occurred in the local process.
!	 	It is the mask for a count performed by dpcount, with a value of
! 		zero returning the loop to the red/black choice, and any greater		
!		value causing a return to the Laplacian convergence loop
!		
!	***********************************************************************	

	use dparlib

	use globals

	use breakup

	use diaggrow

	use treedisp

	use diagnbr

	implicit none

	private

	public dgcomp

      contains

      subroutine dgcomp (pblock, intnbrs, phi, tree, red, blk, stattry)


	use dparlib


	integer :: i, j, k, m, n, q, midrank 
	integer, dimension(1:2,1:3) :: pblock
	integer, parameter :: r4 = selected_real_kind(4)
	integer, parameter :: r8 = selected_real_kind(8)



	integer, dimension( 	pblock(1,1):pblock(2,1), &
				pblock(1,2):pblock(2,2), &
				pblock(1,3):pblock(2,3)), intent(in) :: intnbrs


	real(kind = r8), dimension(pblock(1,1):pblock(2,1), &
				   pblock(1,2):pblock(2,2), &
				   pblock(1,3):pblock(2,3)), intent(in) :: phi


	logical, dimension( 	pblock(1,1):pblock(2,1), &
				pblock(1,2):pblock(2,2), &
				pblock(1,3):pblock(2,3)), intent(inout) :: tree

	logical, dimension(pblock(1,1):pblock(2,1), &
			   pblock(1,2):pblock(2,2), &
			   pblock(1,3):pblock(2,3)), intent(in) :: red, blk


	real(kind = r8), dimension(pblock(1,1):pblock(2,1), &
				   pblock(1,2):pblock(2,2), &
				   pblock(1,3):pblock(2,3)) :: &
		phinbrs, rannarr, breakphi


	logical, dimension(pblock(1,1):pblock(2,1), &
			   pblock(1,2):pblock(2,2), &
			   pblock(1,3):pblock(2,3)) :: nbrs, opens, pass,   &
				edge, face, body, nutree




	logical, dimension(1) :: disch = (/ .false. /)
	integer, intent(inout) :: stattry
	integer :: nbrcount
	real(kind = r4) :: urv 
	real(kind = r4), dimension(:), allocatable :: rannlist    
        real(kind = r8), dimension(:), allocatable :: rannmax
	real(kind = r8) :: nuphi
        real, parameter :: cutoff = 0.1000_r8
	integer :: globnbrcount
	integer :: i, j, k
	integer :: path


  stat: do

	disch(1) = .false.

	call dp_uni(urv)
		if(urv .ge. 0.500) then
			opens = red
!!jgh                     print *, 'red' 
		else
			opens = blk
!!jgh                     print *, 'black'
		end if

		
	edge = .false.
	face = .false.
	body = .false.

			!! Find the position array "nbrs"
			!! corresponding to (intnbrs .gt. 0)
	if(rank .eq. 0) print *, 'Finding nbrs from intnbrs'

 	nbrs = .false.
   		do i = pblock(1,1), pblock(2,2)
			do j = pblock(1,2), pblock(2,2)
				do k = pblock(1,3), pblock(2,3)                    
					if((intnbrs(i,j,k) .ge. 1) .and. &		
						(intnbrs(i,j,k) .le. 6)) then
						nbrs(i,j,k) = .true.
						edge(i,j,k) = .true.
					else if((intnbrs(i,j,k) .ge. 7) .and.         &
						(intnbrs(i,j,k) .le. 18)) then
						nbrs(i,j,k) = .true.
 						face(i,j,k) = .true.
					else if(intnbrs(i,j,k) .ge. 19) then
						nbrs(i,j,k) = .true.
 						body(i,j,k) = .true.
					else if(intnbrs(i,j,k) .eq. 0) then
						nbrs(i,j,k) = .false.
					end if
				end do
			end do
		end do 
	where(nbrs .and. opens) 
                pass = .true.
	elsewhere
		pass = .false.
	end where

        stattry = stattry + 1


!	if(rank .eq. 1)  call shownbrs(pblock, nbrs)

	where(pass .and. edge) phinbrs = phi
	where(pass .and. face) phinbrs = (1.0/1.414)*phi
	where(pass .and. body) phinbrs = (1.0/1.732)*phi
	where(.not. pass) phinbrs = 0.0

	globnbrcount = dp_count(pass)
!!jgh	 print *, 'globnbrcount = ', globnbrcount      
!!jgh	 print *, 'maxval(phinbrs) = ', maxval(phinbrs)
   	 nbrcount = count(pass)
!!jgh		print *, 'nbrcount = ', nbrcount, 'on', rank		
		if(globnbrcount .gt. 0) then

			allocate(rannlist(nbrcount))
       		        allocate(rannmax(nbrcount))
!        if(rank .eq. 0) print *, "nbrcount = ", nbrcount

	call dp_uni_1d(rannlist)
        rannmax = rannlist

	rannarr = 1.0
  rannarr = unpack(rannmax, mask = (phinbrs .gt. cutoff), field = rannarr)
             nutree = .false.
	where(phinbrs .gt. rannarr) nutree = .true.
         print *, 'dp_count of nutree = ' , dp_count(nutree)  
	tree = nutree .or. tree

    if(dp_count(nutree) .eq. 0) then
      if(rank .eq. 0) print *, "No survivors"

                 deallocate(rannlist)
		 deallocate(rannmax) 
                 cycle

      else if(dp_count(nutree) .gt. 0) then
		breakphi = 0.0
		where(nutree) breakphi = phinbrs 
	!!jgh   print *, "maxval(breakphi) = ", maxval(breakphi) 



	do q = 0,TOTAL_PROCS-1

	    if(rank .eq. q) then

		open(unit = 9, file = "streamer", &
		position = "append", status = "old")

                do i = pblock(1,1), pblock(2,1)
               	   do j = pblock(1,2), pblock(2,2)
                      do k = pblock(1,3), pblock(2,3)
      			 
                        if(breakphi(i,j,k) .gt. rannarr(i,j,k)) then 
			  nuphi = breakphi(i,j,k)
				path = intnbrs(i,j,k)				
			  call writestrm(i, j, k, nuphi, stattry, path)
                        else
                          cycle
                        end if

                      end do
		   end do
		end do

		close(unit= 9)
 	    end if

	    call dp_sync()

	end do

        deallocate(rannlist)
        deallocate(rannmax)    


      end if
		
    else if(globnbrcount .eq. 0)  then
      if (rank .eq. 0) print *, "No neighbors eligible"
			cycle
    end if

    if(dp_count(nutree) .gt. 0) disch(1) = .true.	

!   print*, "rank ", rank, "   dp_count(disch) = ", dp_count(disch)
!   print *, "rank ", rank, "   disch = ", disch

    if(dp_count(disch) .eq. TOTAL_PROCS) then 
	exit stat
    else if(dp_count(disch) .lt. TOTAL_PROCS) then
	cycle
    end if	

  end do stat
 
	end subroutine dgcomp

	
	end module diagcomp
         	
