	module diaggrow

	! initializes the arrays phi,tree,cathode,nbrs.
	! initializes integer variables lpltry,stattry.
	! assigns red and black arrays from row and col.
        ! opens file labeled "streamer" to receive breakdown-tree data.
	! initializes anode components of the tree array.
        ! Initializes cathode array.

	use dparlib

	use globals

	use breakup

	implicit none

	private
	public gradient, grinit, writestrm

	contains                         ! initialization for this run

	subroutine gradient (pblock, phi)
					! allocates and initializes phi

	integer, dimension(1:2,1:3) :: pblock
	integer :: i, j, k

	real(kind = r8), dimension ( 	pblock(1,1):pblock(2,1),   &
			   		pblock(1,2):pblock(2,2),   &
			   		pblock(1,3):pblock(2,3) ), &
			intent(inout) :: phi

	real(kind = r8) :: del

	! Create uniform-gradient voltage field anode-to-cathode

	do i = pblock(1,1), pblock(2,1)
		do j = pblock(1,2), pblock(2,2)
			do k = pblock(1,3),pblock(2,3)
                		del = real(aglob-1)
				phi(i,j,k) = (i-1) * (1.0/del)
			end do
		end do
	end do

	end subroutine gradient



        subroutine grinit (pblock, tree, cathode)
					! allocates and initializes tree
					! allocates and assigns cathode
                                        ! within the individual process
                                        ! block bounds

	integer :: a, b, c, i
	integer, dimension(1:2,1:3) :: pblock
	logical, dimension ( 	pblock(1,1):pblock(2,1),   &
			   	pblock(1,2):pblock(2,2),   &
			   	pblock(1,3):pblock(2,3) ), &
					intent(inout) :: tree, cathode

	integer :: local_anode_bounds(2,3)

	if (pblock (1,1) .eq. 1) then
	  	tree(pblock(1,1),:,:) = .true.           ! anode plate
	end if

                                        ! position anode needle
	do i = 1, 3
		local_anode_bounds(1,i) = &
			max (global_anode_bounds(1,i), pblock(1,i))
		local_anode_bounds(2,i) = &
			min (global_anode_bounds(2,i), pblock(2,i))
	end do

        tree (  local_anode_bounds(1,1):local_anode_bounds(2,1), &
        	local_anode_bounds(1,2):local_anode_bounds(2,2), &
        	local_anode_bounds(1,3):local_anode_bounds(2,3) ) = .true.
 
	cathode(ubound(cathode,1),:,:) = .true.    !  cathode plate 

					! Test opening for empty file 'streamer'
	do i = 1, TOTAL_PROCS		! The file opening is staggered.
		if(rank .eq. (i-1)) then		
			open(unit = 9, file = "dgstreamer", &
				position = "append", status = "old")
			print *, "streamer access o.k. on  ", rank
			close(unit = 9)
		end if
		call dp_sync()
	end do
	

	print *, "End of grinit on rank ", rank
	end subroutine grinit

	subroutine writestrm (i, j, k, phival, stattry, pathnum)
  
        implicit none 
        integer, intent(in) :: i, j, k
        integer, parameter :: r8 = selected_real_kind(8)
	real(kind = r8) :: phival
        integer, intent(in) :: stattry, pathnum

                                ! Writes to a data list of positions on the
                                ! streamer tree, with the field strength,
                                ! statcomp iteration no. since start.

        WRITE (9, fmt = 101) i, j, k, phival, stattry, pathnum
  101   format (3i5,'    ',f6.4,' ',i9, i3) 
        
        end subroutine writestrm



	end module diaggrow
