     module lpl

     use dparlib

     use globals
	
     use breakup
	
     use treedisp

     implicit none

     private 
     public laplace, lplinit


     contains

     subroutine laplace( pblock, phi, tree, cathode, red, blk)
	!! Solve Laplace's equation using Gauss-Seidel over-relaxation

	integer, dimension(1:2,1:3) :: pblock
	integer, parameter :: r8 = selected_real_kind(8)  ! request r8

	real(kind = r8), dimension( 	pblock(1,1):pblock(2,1), &
					pblock(1,2):pblock(2,2), &
					pblock(1,3):pblock(2,3) ), &
							intent(inout) :: phi

	real(kind = r8), dimension( 	pblock(1,1):pblock(2,1), 	&
					pblock(1,2):pblock(2,2), 	&
					pblock(1,3):pblock(2,3) ) ::	&
		blkphi, redphi, upblk, upred, rdav, blkav, difb, difr

	logical, dimension( 	pblock(1,1):pblock(2,1), 	&
				pblock(1,2):pblock(2,2), 	&
				pblock(1,3):pblock(2,3) ), intent (in) ::  &
		tree, cathode, red, blk


	logical, dimension(1) :: cnv = .false.      ! cnv shows when block has converged
	integer :: lpltry                           ! lpltry counts convergence loops

                                	!! This variable is the amount of OVER-relaxation
	real, parameter :: w = 1.2000_r8

	real, parameter :: epsilon = 0.0050_r8      ! convergence criterion
	real, parameter :: dvsr = 0.16666666_r8


          where(tree) phi = 0.00000
          where(cathode) phi = 1.0000

          lpltry = 0     ! Counter for the red-black convergence loop;


	cnv(1) = .false.  

	do  !! until the process has converged

	  where(tree) phi = 0.00000 
	  where(cathode) phi = 1.00000

	  upblk = 0.0000
	  upred = 0.0000
	  difb = 0.0000
	  difr = 0.0000

          where(red) blkphi = 0.0000
          where(blk) blkphi = phi
          where(blk) redphi = 0.0000
          where(red) redphi = phi

	  !!
	  !! First, new values for phi are calculated at the black
	  !! sites based on the current values at the red sites.
	  !! Note the use of the over-relaxation parameter w.
	  !!



          rdav = dp_cshift(redphi, +1, 2) + dp_cshift(redphi, -1, 2) &
                   + dp_cshift(redphi, +1, 1) + dp_cshift(redphi, -1, 1) &
                     + dp_cshift(redphi, +1, 3) + dp_cshift(redphi, -1, 3)
       
          where(.not.(tree .or. cathode)) upblk = w * ((dvsr * rdav) - blkphi)
            difb = abs(upblk)
          where(.not. (tree .or. cathode)) blkphi = upblk + blkphi

	  !!
	  !! Now calculate new values of phi at the red sites based on
	  !! the NEW values at the black sites.  The use of the newly 
	  !! calculated values is what makes this the Gauss-Siedel method.
	  !!

          blkav = dp_cshift(blkphi, +1, 2) + dp_cshift(blkphi, -1, 2) &
                   + dp_cshift(blkphi, +1, 1) +dp_cshift(blkphi, -1, 1) & 
                     + dp_cshift(blkphi, +1, 3) + dp_cshift(blkphi, -1, 3) 


          where(.not.(tree .or. cathode)) upred = w * ((dvsr * blkav) - redphi)
          difr = abs(upred)

          where(.not. (tree .or. cathode)) redphi = upred + redphi

          where(.not. (tree .or. cathode)) phi = blkphi + redphi


          lpltry = lpltry + 1



	  
	  if(rank .eq. 0) print *, 'lpltry =  ', lpltry    


	  !! Now check for convergence.

	  if((maxval(difr) .lt. epsilon) &
                   .and. (maxval(difb) .lt. epsilon)) then  
                                              cnv(1) = .true.
	  end if


	  if(dp_count(cnv) .lt. TOTAL_PROCS) then 
               cycle 
            else if(dp_count(cnv) .eq. TOTAL_PROCS) then
		cnv(1) = .false.
		exit
	  end if


	  if((dp_maxval(difr) .lt. epsilon) .and. &
			(dp_maxval(difb) .lt. epsilon)) then  
		exit
	  end if



	

	end do

             


	return
	end subroutine laplace

     subroutine lplinit(pblock, phi, tree, cathode, red, blk)

	integer, dimension(1:2,1:3) :: pblock
	integer, parameter :: r8 = selected_real_kind(8)  ! request r8

	real(kind = r8), dimension( 	pblock(1,1):pblock(2,1), &
					pblock(1,2):pblock(2,2), &
					pblock(1,3):pblock(2,3) ), &
		intent(inout) :: phi



	real(kind = r8), dimension( 	pblock(1,1):pblock(2,1), &
					pblock(1,2):pblock(2,2), &
					pblock(1,3):pblock(2,3) ) :: &
		blkphi, redphi, upblk, upred, rdav, blkav, difb, difr


	logical, dimension( 	pblock(1,1):pblock(2,1), &
				pblock(1,2):pblock(2,2), &
				pblock(1,3):pblock(2,3) ), intent(in) :: &
		tree, cathode, red, blk


	logical, dimension(1) :: cnv = .false.
	integer :: lpltry
	real, parameter :: w = 1.2000_r8
	real, parameter :: epsilon = 0.0050_r8
	real, parameter :: dvsr = 0.16666666_r8



          where(tree) phi = 0.00000
          where(cathode) phi = 1.0000

          lpltry = 0     ! Counter for the red-black convergence loop;

	 print *, "starting lpl init loops on rank: ", rank

	cnv(1) = .false.  

	do 


	where(tree) phi = 0.00000 
	where(cathode) phi = 1.00000

	upblk = 0.0000
	upred = 0.0000
	difb = 0.0000
	difr = 0.0000

          where(red) blkphi = 0.0000
          where(blk) blkphi = phi
          where(blk) redphi = 0.0000
          where(red) redphi = phi


          rdav = dp_cshift(redphi, +1, 2) + dp_cshift(redphi, -1, 2) &
                   + dp_cshift(redphi, +1, 1) + dp_cshift(redphi, -1, 1) &
                     + dp_cshift(redphi, +1, 3) + dp_cshift(redphi, -1, 3)
       
          where(.not.(tree .or. cathode)) upblk = w * ((dvsr * rdav) - blkphi)
            difb = abs(upblk)
          where(.not. (tree .or. cathode)) blkphi = upblk + blkphi

          blkav = dp_cshift(blkphi, +1, 2) + dp_cshift(blkphi, -1, 2) &
                   + dp_cshift(blkphi, +1, 1) + dp_cshift(blkphi, -1, 1) & 
                     + dp_cshift(blkphi, +1, 3) + dp_cshift(blkphi, -1, 3) 


          where(.not.(tree .or. cathode)) upred = w * ((dvsr * blkav) - redphi)
            difr = abs(upred)
          where(.not. (tree .or. cathode)) redphi = upred + redphi

          where(.not. (tree .or. cathode)) phi = blkphi + redphi


          lpltry = lpltry + 1


	 if(rank .eq. 0) print *, 'lpltry =  ', lpltry    

					! lplinit runs for 200 loops
	    if(lpltry .ge. 200) then
                       cnv(1) = .true.
	    end if


	if(dp_count(cnv) .lt. TOTAL_PROCS) then 
               cycle 
            else if(dp_count(cnv) .eq. TOTAL_PROCS) then
		cnv(1) = .false.
		exit
	end if
	

         end do

             
         end subroutine lplinit

         end module lpl  
