      program cadbkl  


 
	use dparlib

	use globals     ! declares global variables, queries for seeds,
			! and initializes random no. generator.


	use breakup     ! makes allocations of partioned arrays.
	
	use growinit    ! initializes arrays & parameters

        use nbr         ! finds all neighbors to existing tree
  
        use nbrdisp     ! displays a plane of neighbors

        use lpl         ! Laplacian solver

	use treedisp    ! displays a plane of the tree

        use fastbkl       ! statistical test for growth

	implicit none

	!!!!!!!!!!!!!!!!!!!


	integer ierr
	integer, save :: stattry = 0
	integer lim
	integer, dimension(MAX_PROCS) :: seedvec      ! Seeds for dp_uni
	real(kind = r8) :: fseed

	integer stat
	integer :: pwr

	!! Read in the size of the data array and the process array
	open (unit=10, file="breakdown.in", status="old", iostat=stat)
	if (stat .ne. 0) then
		print *, "Error opening file system_size."
		stop
	endif

	!! read global data array dimensions
	read (10, *) aglob, bglob, cglob

	!! process array dimensions
	read (10, *) N_PROC_ROWS, N_PROC_COLS, N_PROC_DKS

	!! location of the anode in the global data array
	read (10, *) global_anode_bounds (1,1), global_anode_bounds(2,1)
	read (10, *) global_anode_bounds (1,2), global_anode_bounds(2,2)
	read (10, *) global_anode_bounds (1,3), global_anode_bounds(2,3)

	!! exponent of the desired power law in 'fastthree' subroutine
	read (10, *) pwr
	close (10)


	!! Find out how many processes we actually have.
	call dp_get_num_procs (TOTAL_PROCS, ierr)

	!! Check that the actual number corresponds to the number
	!! specified by the process array dimensions.
	if (TOTAL_PROCS .ne. (N_PROC_ROWS*N_PROC_COLS*N_PROC_DKS)) then
		print *, "Total number of available processes (", &
			TOTAL_PROCS, ") does not match specified process"
		print *, "    array dimensions: ",   &
				N_PROC_ROWS, N_PROC_COLS, N_PROC_DKS
		stop
	endif



	!! Make sure that the data array size is divisible by the process
	!! array size.
	if ( &
		(MOD (aglob, N_PROC_ROWS) .ne. 0) .or.  &
		(MOD (bglob, N_PROC_COLS) .ne. 0) .or.  &
		(MOD (cglob, N_PROC_DKS ) .ne. 0)       ) then

		print *, "Data array size (", aglob, bglob, cglob, &
			") is not divisible by process array size (", &
			N_PROC_ROWS, N_PROC_COLS, N_PROC_DKS, ")."
		stop
	endif

		print *, 'process instructions received at  ', rank

	call dp_initialize(3, (/N_PROC_ROWS, N_PROC_COLS, N_PROC_DKS/), &
		 rank, proc_coords, ierr)

		print *, 'dpinitialize completed at  ', rank

	call dp_query_layout (3, (/aglob, bglob, cglob/), 1, &
						pblk_size, pblock, ierr)

	call alloc

		print *, 'allocation completed on  ', rank

        call user_query(seedvec(1:TOTAL_PROCS))


	fseed = dp_uni_init(seedvec(1:TOTAL_PROCS))

	call gradient (pblock, phi)

        call grinit (pblock, tree, cathode)


	if(rank .eq. TOTAL_PROCS/2) call showtree (pblock, tree)

	call lplinit(pblock, phi, tree, cathode, red, blk)

	call nbrset(pblock, tree, cathode, nbrs)

	if(rank .eq. TOTAL_PROCS/2) call shownbrs &
					(pblock, nbrs)


	if(rank .eq. 0) print *, "starting to call laplace routine"

	do 


          call laplace(pblock, phi, tree, cathode, red, blk)
       
          call nbrset(pblock, tree, cathode, nbrs)
    
               call fbkl (pblock, pwr, nbrs, phi, tree, red, blk, stattry)

          if (rank .eq. 0) print *, "stattry =                     ", stattry   

	  if ((pblock(1,1).le.(aglob-1)) .and. ((aglob-1).le.pblock(2,1)) ) then
	  	fin = tree((aglob-1):(aglob-1),:,:)
	  else
		fin = .false.
	  endif

	  if(dp_any(fin))  then
		print *, "exiting from process ", rank 
		exit
	  end if
           
        end do

 	call dp_finalize(ierr)


	stop
      end program cadbkl 
