          module singlcomp

!	***********************************************************************
!		This is a version of the statistical growth algorithm which uses	
!		a linear statistical comparison against a uniform random number
! 		distribution.		
!		The 1-dimensional, 1-component array disch() keeps track of		
!		whether a discharge breakdown has occurred in the local process.
!	 	It is the mask for a count performed by dp_count, with a value of
! 	        less than the number of processes returning the loop to the 
!                red/black choice, and when the numbers agree,		
!		causing a return to the Laplacian convergence loop
!		Commented print statements can be used to track the routine.	
!	***********************************************************************	

	use dparlib

	use globals

	use breakup

	use growinit

	use treedisp

	implicit none

	private

	public singlecomp

      contains

      subroutine singlecomp (pblock, nbrs, phi, tree, red, blk, stattry)


	use dparlib


	integer :: m, n, q 
	integer, dimension(1:2,1:3) :: pblock
	integer, parameter :: r4 = selected_real_kind(4)
	integer, parameter :: r8 = selected_real_kind(8)



	logical, dimension( 	pblock(1,1):pblock(2,1), &
				pblock(1,2):pblock(2,2), &
				pblock(1,3):pblock(2,3)), intent(in) :: nbrs


	real(kind = r8), dimension(pblock(1,1):pblock(2,1), &
				   pblock(1,2):pblock(2,2), &
				   pblock(1,3):pblock(2,3)), intent(in) :: phi


	logical, dimension( 	pblock(1,1):pblock(2,1), &
				pblock(1,2):pblock(2,2), &
				pblock(1,3):pblock(2,3)), intent(inout) :: tree

	logical, dimension(pblock(1,1):pblock(2,1), &
			   pblock(1,2):pblock(2,2), &
			   pblock(1,3):pblock(2,3)), intent(in) :: red, blk


	real(kind = r8), dimension(pblock(1,1):pblock(2,1), &
				   pblock(1,2):pblock(2,2), &
				   pblock(1,3):pblock(2,3)) :: &
		phinbrs, rannarr, breakphi


	logical, dimension(pblock(1,1):pblock(2,1), &
			   pblock(1,2):pblock(2,2), &
			   pblock(1,3):pblock(2,3)) :: opens, pass, nutree


	logical, dimension(1) :: disch = (/ .false. /)
	integer, intent(inout) :: stattry
	integer :: nbrcount
	real(kind = r4) :: urv 
	real(kind = r4), dimension(:), allocatable :: rannlist    
        real(kind = r8), dimension(:), allocatable :: rannmax
	real(kind = r8) :: nuphi
        real, parameter :: cutoff = 0.0150_r8  ! The important voltage threshold.
	integer :: globnbrcount
	integer :: i, j, k


  stat: do

	disch(1) = .false.

	call dp_uni(urv)
		if(urv .ge. 0.500) then
			opens = red
		else
			opens = blk
		end if

	where(nbrs .and. opens) 
                pass = .true.
	elsewhere
		pass = .false.
	end where

        stattry = stattry + 1

	if((rank .eq. 0) .and. (mod(stattry,10) .eq. 0))  & 
				call showtree(pblock, tree)

        where(pass .and. (phi .gt. cutoff))
		phinbrs = phi       

	elsewhere
		phinbrs = 0.0
	end where

	globnbrcount = dp_count(phinbrs .gt. cutoff)

!!	 print *, 'globnbrcount = ', globnbrcount      
!!	 print *, 'maxval(phinbrs) = ', maxval(phinbrs)

   	 nbrcount = count(phinbrs .gt. cutoff)

!!		print *, 'nbrcount = ', nbrcount, 'on', rank	
	
		if(globnbrcount .gt. 0) then

			allocate(rannlist(nbrcount))
       		        allocate(rannmax(nbrcount))

	call dp_uni_1d(rannlist)
        rannmax = rannlist

	rannarr = 1.0

  rannarr = unpack(rannmax, mask = (phinbrs .gt. cutoff), field = rannarr)
             nutree = .false.
	where(phinbrs .gt. rannarr) nutree = .true.

         print *, 'dp_count of nutree = ' , dp_count(nutree)  

	tree = nutree .or. tree

    if(dp_count(nutree) .eq. 0) then
      if(rank .eq. 0) print *, "No survivors"

                 deallocate(rannlist)
		 deallocate(rannmax) 
                 cycle

      else if(dp_count(nutree) .gt. 0) then
		breakphi = 0.0
		where(nutree) breakphi = phinbrs
 
	!!jgh   print *, "maxval(breakphi) = ", maxval(breakphi) 



	do q = 0,TOTAL_PROCS-1

	    if(rank .eq. q) then

		open(unit = 9, file = "streamer", &
		position = "append", status = "old")

                do i = pblock(1,1), pblock(2,1)
               	   do j = pblock(1,2), pblock(2,2)
                      do k = pblock(1,3), pblock(2,3)
      			 
                        if(breakphi(i,j,k) .gt. rannarr(i,j,k)) then 
			  nuphi = breakphi(i,j,k) 
			  call writestrm(i, j, k, nuphi, stattry)
                        else
                          cycle
                        end if

                      end do
		   end do
		end do

		close(unit= 9)
 	    end if

	    call dp_sync()

	end do

        deallocate(rannlist)
        deallocate(rannmax)    


      end if
		
    else if(globnbrcount .eq. 0)  then
      if (rank .eq. 0) print *, "No neighbors eligible"
			cycle
    end if

    if(dp_count(nutree) .gt. 0) disch(1) = .true.	

!   print*, "rank ", rank, "   dp_count(disch) = ", dp_count(disch)
!   print *, "rank ", rank, "   disch = ", disch

    if(dp_count(disch) .eq. TOTAL_PROCS) then 
	exit stat
    else if(dp_count(disch) .lt. TOTAL_PROCS) then
	cycle
    end if	

  end do stat

	end subroutine singlecomp

	
	end module singlcomp
         	
